'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var bluebird = require('bluebird');
var fs = bluebird.promisifyAll(require('fs'));
var path = require('path');
var detective = require('detective');
var fetch = require('node-fetch');
var builtins = require('builtin-modules');

var infer = {
	main: function main(id, dir, repo) {
		return fs.readdirAsync(dir).then(function (files) {
			return files.filter(function (file) {
				return file.endsWith('.js');
			});
		}).then(function (jsFiles) {
			return jsFiles[0];
		});
	},

	version: function version(id, dir, repo) {
		return repo.getBranchCommit('master').then(function (master) {
			return new bluebird(function (resolve, reject) {
				var history = master.history();
				history.on('end', resolve);
				history.on('error', reject);
				history.start();
			}).then(function (commits) {
				return '1.' + (commits.length - 1) + '.0';
			});
		});
	},

	name: function name(id, dir, repo) {
		return infer.main(id, dir, repo).then(function (main) {
			return path.basename(main, '.js');
		}).then(function (name) {
			return fetch('https://api.github.com/gists/' + id).then(function (r) {
				return r.json();
			}).then(function (gist) {
				return '@' + gist.owner.login + '/' + name;
			});
		});
	},

	author: function author(id, dir, repo) {
		return repo.getBranchCommit('master').then(function (commit) {
			return commit.author();
		}).then(function (author) {
			return author.name() + ' <' + author.email() + '>';
		});
	},

	dependencies: function dependencies(id, dir, repo) {
		return infer.main(id, dir, repo).then(function (main) {
			return path.resolve(dir, main);
		}).then(function (mainPath) {
			return fs.readFileAsync(mainPath, 'utf8').then(function (src) {
				return detective(src);
			}).then(function (packages) {
				return stars(packages.filter(function (p) {
					return builtins.indexOf(p) === -1;
				}).map(actualPackageName));
			});
		});
	}
};

var actualPackageName = function actualPackageName(req) {
	return req.split('/', req[0] === '@' ? 2 : 1).join('/');
};

var stars = function stars(deps) {
	return arraysToObj(deps, arrayN(deps.length, '*'));
}; //yolo
var arrayN = function arrayN(n, x) {
	return n <= 0 ? [] : [x].concat(arrayN(n - 1, x));
};
var arraysToObj = function arraysToObj(xs, ys) {
	return xs.reduce(function (o, x, i) {
		return (o[x] = ys[i], o);
	}, {});
};

module.exports = function (id, dir, repo) {
	var keys = _Object$keys(infer);
	return bluebird.all(keys.map(function (k) {
		return infer[k](id, dir, repo);
	})).then(function (values) {
		return arraysToObj(keys, values);
	});
};
